""" Tests for the workflow service """

import json
from typing import List

# pylint: disable=E0401, R0201, W0212
from unittest.mock import patch

import pytest

from workspaces.workflow.schema import WorkflowRequest, WorkflowRequestFile
from workspaces.workflow.services.workflow_info import WorkflowInfo
from workspaces.workflow.services.workflow_service import WorkflowService

pytest_plugins = ["testing.utils.conftest"]

"""
Tests for WorkflowService
"""


@pytest.mark.usefixtures("mock_workflow_info", "mock_workflow_service", "test_router")
class TestWorkflowService:
    """Our test class"""

    requests = []

    @pytest.mark.skip(
        "Irreparably broken. Tries to use uninitialized fields of WorkflowRequest."
        "Probably safe to ignore as this is also tested in test_workflow_info.py"
    )
    def test_create_workflow_request(self, mock_workflow_info: WorkflowInfo):
        """
        Is a workflow request created as expected from workflow info?

        :param mock_workflow_info:
        :return:
        """
        request = mock_workflow_info.create_workflow_request(
            mock_workflow_info.lookup_workflow_definition("null"),
            json.loads('{"argument":"-g"}'),
        )
        self.requests.append(request)
        assert request in self.requests

    def test_attach_file_to_request(
        self,
        mock_workflow_info: WorkflowInfo,
        mock_workflow_requests: List[WorkflowRequest],
    ):
        """
        Is a file attached to workflow requests as expected?

        :param mock_workflow_info:
        :param mock_workflow_requests:
        :return:
        """
        request = mock_workflow_requests[0]
        test_filename = "test.txt"
        test_content = b"testing file additions"
        file = mock_workflow_info.save_file(
            request=request, filename=test_filename, content=test_content
        )
        mock_workflow_info.session.add.assert_called_with(file)
        assert mock_workflow_info.session.flush.call_count == 1

    def test_prepare_files_for_condor(self, mock_workflow_service: WorkflowService):
        """
        Make sure files are prepared for Condor.

        :param mock_workflow_service:
        :return:
        """
        files = [
            WorkflowRequestFile(
                workflow_request_id=1,
                filename="null.condor",
                content=b"executable = null\n"
                + b"arguments = {{arguments}}\n"
                + b"error = null.err\n"
                + b"log = condor.log\n\n\n"
                + b"queue",
            ),
            WorkflowRequestFile(
                workflow_request_id=1,
                filename="null.dag",
                content=b"JOB null null.condor",
            ),
            WorkflowRequestFile(
                workflow_request_id=1,
                filename="null.sh",
                content=b"#!/bin/sh\n\n" + b"null $*",
            ),
        ]
        temp = mock_workflow_service._make_temp_directory()
        path = mock_workflow_service._prepare_files_for_condor(files, temp)

        assert path.exists()

    @pytest.mark.skip(
        "fails at workflow_service.py:379 with \"AttributeError: 'WorkflowService' object has no attribute 'update_status'\""
    )
    def test_update_request_complete(
        self,
        mock_workflow_service: WorkflowService,
        mock_workflow_requests: List[WorkflowRequest],
    ):
        """
        TODO some fine day: is this test even necessary if other tests pass?
        Make sure request status really gets updated.

        :param: a pretend workflow info

        :return:
        """

        request = mock_workflow_requests[0]
        message = {
            "subject": {
                "workflow_request_id": request.workflow_request_id,
                "type": "WorkflowRequest",
            },
            "type": "workflow-complete",
        }
        assert request.state == "Ready"

        with patch("workspaces.workflow.schema.WorkflowRequest.update_status") as mock_update:
            mock_workflow_service.on_workflow_event(**message)
            mock_update.assert_called_once()
            assert request.state == "Complete"
